#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "type.h"
#include "cd.h"
#include "Win32Def.h"
#include "resource.h"
#include "psemu.h"
#include "locals.h"

typedef struct {
    char FileName[32];
} PluginType;

static unsigned int PlugNum = 0;
static PluginType   PlugList[16];
static HANDLE       hinstance = (HANDLE)-1;
static FPSEWin32   *Info;

char         LocalPath[256];

static FPSEWin32About info = {
    FPSE_CDROM,
    VERSION & 0xFF,
    VERSION >> 8,
    0,
    "LDChen",
    "CDPsEmu",
    "This Plugin allows to use PsemuPro Cdr plugin with FPSE."
};

int LoadPlugin(char *name)
{
    CdrDLL = LoadLibrary(name);

    if (CdrDLL < (HMODULE)(32)) return FPSE_ERR;

    CDRinit     = (long *)GetProcAddress(CdrDLL,"CDRinit");
    CDRshutdown = (long *)GetProcAddress(CdrDLL,"CDRshutdown");
    CDRopen     = (long *)GetProcAddress(CdrDLL,"CDRopen");
    CDRclose    = (long *)GetProcAddress(CdrDLL,"CDRclose");

    CDRgetTN    = (long *)GetProcAddress(CdrDLL,"CDRgetTN");
    CDRgetTD    = (long *)GetProcAddress(CdrDLL,"CDRgetTD");

    CDRreadTrack = (long *)GetProcAddress(CdrDLL,"CDRreadTrack");
    CDRgetBuffer = (long *)GetProcAddress(CdrDLL,"CDRgetBuffer");

    CDRconfigure = (long *)GetProcAddress(CdrDLL,"CDRconfigure");
    CDRtest      = (long *)GetProcAddress(CdrDLL,"CDRtest");
    CDRabout     = (long *)GetProcAddress(CdrDLL,"CDRabout");

    PSEgetLibName    = (long *)GetProcAddress(CdrDLL,"PSEgetLibName");
    PSEgetLibType    = (long *)GetProcAddress(CdrDLL,"PSEgetLibType");
    PSEgetLibVersion = (long *)GetProcAddress(CdrDLL,"PSEgetLibVersion");

    return FPSE_OK;
}

static void ScanPlugins(HWND hwnd)
{
    WIN32_FIND_DATA data;
    HANDLE          h;
    char            buf[256];
    int             x;

    strcpy(buf,LocalPath);
    strcat(buf,"PSEMU\\*.dll");

    h = FindFirstFile(buf,&data);
    if (h == INVALID_HANDLE_VALUE) return;
    for (;;)
    {
        if (!(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
        {
            strcpy(buf,LocalPath);
            strcat(buf,"PSEMU\\");
            strcat(buf,data.cFileName);
            if (LoadPlugin(buf) == FPSE_OK)
            {
                if (PSEgetLibType() == PSE_LT_CDR)
                {
                    strcpy(PlugList[PlugNum].FileName,data.cFileName);
                    x = PSEgetLibVersion();
                    sprintf(buf,"%s %d.%d",
                            (char *)PSEgetLibName(), (x>>8) & 0xFF, x & 0xFF);
                    SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),
                                CB_ADDSTRING,0,(long)buf);
                    PlugNum++;
                }
                FreeLibrary(CdrDLL);
            }
        }
        if (FindNextFile(h,&data) == FALSE) break;
    }
    FindClose(h);
}


void GetActiveDir(HINSTANCE hInstance)
{
    char *p;

// First get the full path of the application
    GetModuleFileName(hInstance, LocalPath, 256);

// Then trim off the program name leaving just the path to the directory (C:\DIR\DIR\)
   p = LocalPath;
   while(strchr(p,'\\')) {  // while there are more separators
      p = strchr(p,'\\');   // point to next separator
      p++;                  // point to char after last separator
   }
   *p = '\0';               // terminate string
}

BOOL WINAPI
#ifdef __GNUC__
LibMain
#else
DllMain
#endif
(HANDLE hInstance, DWORD fdwReason, PVOID fImpLoad)
{
    hinstance = hInstance;

    GetActiveDir(hInstance);

    return TRUE;
}

BOOL CALLBACK ConfigDialogProc(HWND hwnd, UINT msg, WPARAM wPar, LPARAM lPar)
{
    char buf[256];
    int  number;

    switch (msg) {
    case WM_INITDIALOG:
        ScanPlugins(hwnd);

        if (PlugNum) {
            SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),CB_SETCURSEL,0,0);
        } else {
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),FALSE);
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_CONFIGURE_BUTTON),FALSE);
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_TEST_BUTTON),FALSE);
            EnableWindow((HWND)GetDlgItem(hwnd,IDC_ABOUT_BUTTON),FALSE);
        }
        return TRUE;

    case WM_COMMAND:
        switch (wPar) {
        case IDC_TEST_BUTTON:
        case IDC_ABOUT_BUTTON:
        case IDC_CONFIGURE_BUTTON:
            number = SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),
                                 CB_GETCURSEL,0,0);
            strcpy(buf,LocalPath);
            strcat(buf,"PSEMU\\");
            strcat(buf,PlugList[number].FileName);
            LoadPlugin(buf);
            switch (wPar) {
            case IDC_TEST_BUTTON:
                number = 1;
                if (CDRtest != NULL) number = CDRtest();
                switch (number) {
                case PSE_ERR_SUCCESS:
                    sprintf(buf,"This driver reports that it should work correctly!");
                    break;
                case PSE_CDR_ERR:
                case PSE_ERR_FATAL:
                    sprintf(buf,"This driver can't run on your hardware!");
                default:
                    sprintf(buf,"This is an old style driver and does not support TEST function");
                    break;
                }
                MessageBox(hwnd,buf,"Psemu",MB_OK | MB_ICONINFORMATION);
                break;
            case IDC_ABOUT_BUTTON:
                CDRabout();
                break;
            case IDC_CONFIGURE_BUTTON:
                CDRconfigure();
                break;
            }
            FreeLibrary(CdrDLL);
            return TRUE;
        case IDOK:
            number = SendMessage((HWND)GetDlgItem(hwnd,IDC_COMBOPSE),
                                 CB_GETCURSEL,0,0);
            Info->WriteCfg("CDRPSEMU","PluginName",PlugList[number].FileName);
        case IDCANCEL:
            EndDialog(hwnd,0);
            return TRUE;
        }
        break;
    case WM_DESTROY:
        EndDialog(hwnd,0);
        return TRUE;
    }

    return FALSE;
}

int CD_Configure(UINT32 *par)
{
    Info = (FPSEWin32 *)par;

    DialogBox(hinstance,MAKEINTRESOURCE(IDD_DIALOG),
              Info->HWnd,ConfigDialogProc);
    return FPSE_OK;
}

void CD_About(UINT32 *par)
{
    memcpy( par, &info, sizeof(FPSEWin32About) );
}
